
// Tokamak Tutorial 5
//
// By Adam Dawes, www.adamdawes.com, July 2004


#include <windows.h>
#include <atlbase.h>
#include <stdlib.h>
#include <time.h>

#include "Main.h"
#include "DXUtil.h"


#include <tokamak.h>



#pragma comment(lib,"d3d8.lib")
#pragma comment(lib,"d3dx8.lib")
#pragma comment(lib, "winmm.lib")
#pragma comment(lib, "tokamak.lib")


// Function prototypes

void MessagePump(void);
bool GetFullscreen(void);
bool InitWindow(void);
void KillWindow(void);
bool InitD3D(void);
void KillD3D(void);
bool InitScene(void);
void KillScene(void);
bool InitObjects(void);
void KillObjects(void);
bool InitPhysics(void);
void KillPhysics(void);
void BuildTerrainTriangles(void);
void BuildTerrainDX(neTriangleMesh *triMesh);
void SetTerrainHeights(void);
void TerrainQueryCallback(const neV3 &minBound, const neV3 &maxBound, s32 **candidateTriangles, neTriangle **triangles, neV3 **vertices, s32 *candidateCount, s32 *triangleCount, neRigidBody *rb);
D3DFORMAT FindDisplayMode(void);
bool InitTimer(void);
float GetElapsedTime(void);
bool ValidateDevice(void);
void BuildSphere(IDirect3DVertexBuffer8 **pNewBuff, float fDiameter);
void Render(void);


LRESULT CALLBACK WindowProc(HWND hWnd, UINT Msg, WPARAM WParam, LPARAM LParam);


// Variables for this sourcefile
HINSTANCE ghInstance = NULL;
HWND ghAppWindow = NULL;

LPDIRECT3D8 gD3D=NULL;

D3DPRESENT_PARAMETERS gD3Dpp;

D3DLIGHT8 gLight;

bool gbFullscreen = false;
bool gbFinished = false;


// Global constants
#define APP_NAME "Tokamak Tutorial"
#define WINWIDTH 800
#define WINHEIGHT 600


// Global variables for DirectX
IDirect3DDevice8 *gD3DDevice=NULL;

// Global variables for Tokamak
neSimulator *gSim = NULL;
// The numbers of objects to render in the simulation
#define SPHERECOUNT 3
neRigidBody *gSpheres[SPHERECOUNT];


// Terrain vertex details
#define TERRAINX 15
#define TERRAINZ 15

// The terrain vertex count is simply the number of height nodes we have
#define TERRAIN_VERTEXCOUNT (TERRAINX * TERRAINZ)
// The triangle count is the number of quads (which is one
// less than each dimension of the terrain vertex map) multiplied
// by two (as we need two triangles to make up each quad).
#define TERRAIN_TRIANGLECOUNT ((TERRAINX-1)*(TERRAINZ-1)*2)

// The size (x- and z-axes) and height (y-axis) for the terrain
#define TERRAINSIZE 25.0f
#define TERRAINHEIGHT 10

float gfTerrainHeights[TERRAINX][TERRAINZ];

// Declare a triangle mesh for the floor
neTriangleMesh gTriMesh;
// The array of vertices used by the triangle mesh
neV3 triVertices[TERRAIN_VERTEXCOUNT];
// The array of triangles used by the triangle mesh
neTriangle triData[TERRAIN_TRIANGLECOUNT];
// An array of flags which indicate whether each triangle
// has been used as part of the terrain collision callback
int giTriangleActive[TERRAIN_TRIANGLECOUNT];



// Constants used for the sphere
#define SPHERE_VSLICES 14
#define SPHERE_HSTRIPS 10
#define SPHERE_TRIANGLECOUNT ((SPHERE_HSTRIPS-1) * SPHERE_VSLICES * 2)

#define SPHEREDIAM 1


// Timer variables
bool gbUseHFTimer;
INT64 gCurrentTime;
float gfTimeScale;

// Camera position and scene timer
float gfCameraAngle = 0;
float gfSceneTime = 0;


// Vertex and index buffers
IDirect3DVertexBuffer8 *vbSphere = NULL;
IDirect3DVertexBuffer8 *vbFloor = NULL;
IDirect3DIndexBuffer8 *ibFloor = NULL;



// Application entry point
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR pszCmdLine, int iShow)
{

    CoInitialize(NULL);

	// Store our instance handle
	ghInstance = hInstance;

	// Initialise randomised numbers
	srand( (unsigned)time( NULL ) );

	if (!GetFullscreen())
	{
		OutputDebugString("  User abort\n");
		exit(5);
	}

	if (!InitWindow())
	{
		OutputDebugString("  Window initialisation failed\n");
		exit(5);
	}

	if (!InitD3D())
	{
		OutputDebugString("  Direct3D initialisation failed\n");
		exit(5);
	}

	if (!InitScene())
	{
		OutputDebugString("  Scene initialisation failed\n");
		exit(5);
	}

	if (!InitTimer())
	{
		OutputDebugString("  Timer initialisation failed\n");
		exit(5);
	}

	if (!InitObjects())
	{
		OutputDebugString("  Object initialisation failed\n");
		exit(5);
	}

	if (!InitPhysics())
	{
		OutputDebugString("  Physics initialisation failed\n");
		exit(5);
	}



	// Find the current time so that initialisation time isn't included within the first frame
	GetElapsedTime();


	//Loop until we are closed
	while(!gbFinished)
	{
		//Check for window messages
		MessagePump();

		//Draw our graphics
		Render();
	}


	// Terminate everything
	KillPhysics();
	KillObjects();
	KillScene();
	KillD3D();
	KillWindow();


    CoUninitialize();

	//Exit happily
	return 0;
}


// Process any messages that Windows has sent us
void MessagePump(void)
{
	MSG msg;

	if(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}



// Should we run in full-screen or windowed mode?
bool GetFullscreen(void)
{
	int iResult;
	bool bRet = true;

	iResult = MessageBox(NULL, "Run in fullscreen mode?", APP_NAME, MB_YESNOCANCEL|MB_ICONQUESTION);
	switch(iResult)
	{
		case IDCANCEL:
			bRet = false;
			break;
		case IDNO:
			gbFullscreen = false;
			break;
		case IDYES:
			gbFullscreen = true;
			break;
		case 0:
			OutputDebugString("Couldn't open MessageBox, dying");
			exit(10);
		break;
	}
	
	return bRet;
}



// Initialise a window (full-screen or otherwise) in which our graphics will be displayed.
bool InitWindow(void)
{
	int iWidth, iHeight;
	WNDCLASS WindowClass;
	DWORD dwStyle;

	// Fill in all the fields for the WNDCLASS structure.
	WindowClass.style          = CS_OWNDC;
	WindowClass.cbClsExtra     = 0;
	WindowClass.cbWndExtra     = 0;
	WindowClass.hInstance      = ghInstance;
	WindowClass.hIcon          = LoadIcon(NULL,IDI_APPLICATION);
	WindowClass.hCursor        = LoadCursor(NULL,IDC_ARROW);
	WindowClass.hbrBackground  = (HBRUSH)GetStockObject(BLACK_BRUSH);
	WindowClass.lpszMenuName   = NULL;
	WindowClass.lpszClassName  = APP_NAME;
	WindowClass.lpfnWndProc    = WindowProc;


	// Register the class with windows
	if(!RegisterClass(&WindowClass))
	{
		OutputDebugString("Error registering window class");
		return false;
	}

	// If we're running full screen, we cover the desktop with our window.
	// This isn't necessary, but it provides a smoother transition for the
	// user, especially when we're going to change screen modes.
	if(gbFullscreen)
	{
		iWidth = GetSystemMetrics(SM_CXSCREEN);
		iHeight = GetSystemMetrics(SM_CYSCREEN);
		dwStyle = WS_POPUP;

		ShowCursor(false);		// Hide Mouse Pointer
	}
	else
	{
		//In windowed mode, we just make the window whatever size we need.
		iWidth = WINWIDTH;
		iHeight = WINHEIGHT;
		dwStyle = WS_OVERLAPPED|WS_SYSMENU;
	}


	// Create the window
	ghAppWindow=CreateWindow(APP_NAME, APP_NAME, dwStyle,
			  0, 0, iWidth, iHeight, NULL, NULL,
			  ghInstance, NULL);

	// All ok?
	if(!ghAppWindow)
	{
		OutputDebugString("Error opening window");
		return false;
	}

	// Make sure that our window is visible and has the input focus.
	ShowWindow(ghAppWindow, SW_SHOW);
	UpdateWindow(ghAppWindow);
	SetFocus(ghAppWindow);

	return true;
}



// Terminate the window that we previously opened
void KillWindow(void)
{
	MSG msg;

	//Test if our window is valid
	if(ghAppWindow)
	{
		if(!DestroyWindow(ghAppWindow))
		{
			//We failed to destroy our window, this shouldn't ever happen
			OutputDebugString("   Failed to DestroyWindow\n");
			MessageBox(NULL, "Destroy Window Failed", APP_NAME, MB_OK|MB_ICONERROR|MB_TOPMOST);
		}
		else
		{
			//Clean up any pending messages
			while(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE))
			{
				DispatchMessage(&msg);
			}
		}

		//Set our window handle to NULL just to be safe
		ghAppWindow = NULL;
	}

	ShowCursor(true);		// Show Mouse Pointer

	//Unregister our window, if we had opened multiple windows using this
	//class, we would have to close all of them before we unregistered the class.
	if(!UnregisterClass(APP_NAME, ghInstance))
	{
		OutputDebugString("   Failed to Unregister Window\n");
		MessageBox(NULL, "Unregister Failed", APP_NAME, MB_OK|MB_ICONERROR|MB_TOPMOST);
	}

}





bool InitD3D(void)
{
	HRESULT hr;
	D3DDISPLAYMODE DisplayMode;

	// Create Direct3D8
	gD3D = Direct3DCreate8( D3D_SDK_VERSION );
	if(!gD3D )
	{
		OutputDebugString("  Error getting Direct3D");
		return false;
	}

	// Get the current(desktop) display mode.  This is really only needed if we're running in a window.
	hr = gD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &DisplayMode);
	if(FAILED(hr))
	{
		OutputDebugString("Error getting display mode\n");
		return false;
	}

	// Clear out our D3DPRESENT_PARAMETERS structure.
	ZeroMemory(&gD3Dpp,sizeof(gD3Dpp));

	// Whether we're full-screen or windowed these are the same.
	gD3Dpp.SwapEffect     = D3DSWAPEFFECT_DISCARD;
	gD3Dpp.hDeviceWindow  = ghAppWindow;
	gD3Dpp.BackBufferCount= 1;

	//We need a Z-Buffer so everything will be drawn properly
	gD3Dpp.EnableAutoDepthStencil = TRUE;
	gD3Dpp.AutoDepthStencilFormat = D3DFMT_D16;

	// BackBufferWidth/Height have to be set for full-screen apps
	// BackBufferFormat is the pixel format we want.  In windowed mode we use the same
	// format as the desktop (which we found by using GetAdapterDisplayMode() above).
	// In full-screen we need to find a pixel format we like, see find_16bit_mode()
	// below for more details.
	if(gbFullscreen)
	{
		gD3Dpp.Windowed          = FALSE;
		gD3Dpp.BackBufferWidth   = WINWIDTH;
		gD3Dpp.BackBufferHeight  = WINHEIGHT;
		gD3Dpp.BackBufferFormat  = FindDisplayMode();
	}
	else
	{
		gD3Dpp.Windowed          = TRUE;
		gD3Dpp.BackBufferFormat  = DisplayMode.Format;
	}

	hr = gD3D->CreateDevice(D3DADAPTER_DEFAULT,					//The default adapter, on a multimonitor system
						  D3DDEVTYPE_HAL,						//Use hardware acceleration rather than the software renderer
						  ghAppWindow,
						  D3DCREATE_SOFTWARE_VERTEXPROCESSING,	//Process vertices in software. This is slower than in hardware,
						  &gD3Dpp,								//Our D3DPRESENT_PARAMETERS structure, so it knows what we want to build
						  &gD3DDevice);							//This will be set to point to the new device
	if(FAILED(hr))
	{
		OutputDebugString("Error creating device\n");
		return false;
	}

	// All done
	return true;
}


// Terminate our D3D objects
void KillD3D(void){

	if(gD3DDevice)
	{
		gD3DDevice->Release();
		gD3DDevice=NULL;
	}

	if(gD3D)
	{
		gD3D->Release();
		gD3D=NULL;
	}

}


D3DFORMAT FindDisplayMode(void)
{
	HRESULT hr;

	// First we test for 32-bit modes (these obviously look best)
	hr = gD3D->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_A8R8G8B8, D3DFMT_A8R8G8B8, FALSE);
	if(SUCCEEDED(hr))
	{
		OutputDebugString("D3DFMT_A8R8G8B8\n");
		return D3DFMT_A8R8G8B8;
	}

	hr = gD3D->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, FALSE);
	if(SUCCEEDED(hr))
	{
		OutputDebugString("D3DFMT_X8R8G8B8\n");
		return D3DFMT_X8R8G8B8;
	}

	// No 32-bit modes available so let's try a 16-bit mode
	hr = gD3D->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_R5G6B5, D3DFMT_R5G6B5, FALSE);
	if(SUCCEEDED(hr))
	{
		OutputDebugString("D3DFMT_R5G6B5\n");
		return D3DFMT_R5G6B5;
	}

	// Next try X1R5G5B5.
	hr = gD3D->CheckDeviceType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_X1R5G5B5, D3DFMT_X1R5G5B5, FALSE);
	if(SUCCEEDED(hr))
	{
		OutputDebugString("D3DFMT_X1R5G5B5\n");
		return D3DFMT_X1R5G5B5;
	}

	// Nothing found that we can work with.
	OutputDebugString("Couldn't find a display mode\n");

	KillD3D();
	KillWindow();

	exit(5);

	//Won't actually hit this line since exit() kills us, but it makes the compiler happy.
	return (D3DFORMAT)NULL;
}



// Initialise DirectX ready for us to start rendering
bool InitScene(void)
{
	D3DXMATRIX view_matrix;
	D3DXMATRIX matProj;

	// Set the camera position (it won't change after this)
	// The camera is positioned a short way back from the origin, looking at the origin.
	D3DXMatrixLookAtLH(&view_matrix,&D3DXVECTOR3( 0.0f, 0.0f, -23.0f ),
							   &D3DXVECTOR3( 0.0f, 0.0f, 0.0f ),
							   &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));
	gD3DDevice->SetTransform(D3DTS_VIEW,&view_matrix);

	// The Projection matrix won't change either, so we set it now and never touch it again.
	D3DXMatrixPerspectiveFovLH(&matProj, //Result Matrix
								  D3DX_PI/4,//Field of View, in radians. (PI/4) is typical
								  ((float)WINWIDTH / (float)WINHEIGHT),     //Aspect ratio
								  1.0f,     //Near view plane
								  100.0f ); // Far view plane
	gD3DDevice->SetTransform( D3DTS_PROJECTION, &matProj );

	// Set the device state
	gD3DDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

	// Zero out our light structure and fill in the fields we need
	ZeroMemory( &gLight, sizeof(D3DLIGHT8) );
	gLight.Type=D3DLIGHT_DIRECTIONAL;
	gLight.Diffuse.r = gLight.Diffuse.g = gLight.Diffuse.b = 1.0f;
	gLight.Range=1000.0f;
	gLight.Direction=D3DXVECTOR3(-1, -1, 1);
	gD3DDevice->SetLight( 0, &gLight );
	gD3DDevice->LightEnable( 0, TRUE);

	//Ambient light is the default lighting level of the area.  Anything not hit
	//by our light will have this level of lighting.
	gD3DDevice->SetRenderState(D3DRS_AMBIENT,0x00202020);

	// Turn on lighting.
	gD3DDevice->SetRenderState(D3DRS_LIGHTING,TRUE);

	// Set the COLOROP state to take colour values from the texture data
	gD3DDevice->SetTextureStageState(0,D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
	gD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG1, D3DTA_TEXTURE);

	// Disable alpha blending for extra speed
	gD3DDevice->SetTextureStageState(0,D3DTSS_ALPHAOP,   D3DTOP_DISABLE);

	// Set the magnification filter to Linear for good-quality filtering
	gD3DDevice->SetTextureStageState(0,D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	gD3DDevice->SetTextureStageState(0,D3DTSS_MINFILTER, D3DTEXF_LINEAR);

	// Set the custom vertex shader that we've defined
	gD3DDevice->SetVertexShader(D3D8T_CUSTOMVERTEX);

	// Disable vertex culling so that we can see the floor even when we're underneath it
	gD3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);

	return true;
}




void KillScene(void)
{
}




// Initialise the 3d objects we will be using
bool InitObjects(void)
{
	// Create the Vertex Buffers and copy the vertex data to them
	BuildSphere(&vbSphere, SPHEREDIAM);

	// Create an empty vertex buffer for the floor.
	// We'll use indexed primitives to draw this, so we just need
	// a single vertex for each point in our floor data.
	// We'll add the data to this in the InitPhysics() function.
	BuildVertexBuffer(&vbFloor, NULL, TERRAINX * TERRAINZ);

	// Create an index buffer for the floor.
	// This will require enough triangles to make up the terrain.
	// We'll calculate this as (TERRAINX-1)*(TERRAINZ-1) quads,
	// which results in (TERRAINX-1)*(TERRAINZ-1)*2 triangles and
	// (TERRAINX-1)*(TERRAINZ-1)*2*3 vertices
	BuildIndexBuffer(&ibFloor, NULL, (TERRAINX-1)*(TERRAINZ-1)*2*3);

	return true;
}

// Dispose of the objects we previously initialised
void KillObjects(void)
{
	// Release anything previously set into the vertex/index buffers
	if (ibFloor)
	{
		ibFloor->Release();
		ibFloor=NULL;
	}

	if (vbFloor)
	{
		vbFloor->Release();
		vbFloor=NULL;
	}

	if(vbSphere)
	{
		vbSphere->Release();
		vbSphere=NULL;
	}
}



// Initialise the Tokamak physics engine.
// Here's where the interesting stuff starts.
bool InitPhysics(void)
{
	neGeometry *geom;	// Pointer to a Geometry object which we'll use to define the shape/size of each cube
	neV3 gravity;		// A vector to store the direction and intensity of gravity
	neV3 pos;			// The position of each object
	float fMass;		// The mass of our objects
	neSimulatorSizeInfo sizeInfo;	// SizeInfo stores data about how many objects we are going to model
	float fYPos, fYPosAdd;
	int i;


	// Create and initialise the simulator
	// Tell the simulator how many rigid bodies we have
	sizeInfo.rigidBodiesCount = SPHERECOUNT;
	// Tell the simulator how many animated bodies we have
	sizeInfo.animatedBodiesCount = 0;
	// Tell the simulator how many bodies we have in total
	s32 totalBody = sizeInfo.rigidBodiesCount + sizeInfo.animatedBodiesCount;
	sizeInfo.geometriesCount = totalBody;
	// The overlapped pairs count defines how many bodies it is possible to be in collision
	// at a single time. The SDK states this should be calculated as:
	//   bodies * (bodies-1) / 2
	// So we'll take its word for it. :-)
	sizeInfo.overlappedPairsCount = totalBody * (totalBody - 1) / 2;
	// We're not using any of these so set them all to zero
	sizeInfo.rigidParticleCount = 0;
	sizeInfo.constraintsCount = 0;
	// No need to tell Tokamak about our terrain nodes, the sizeInfo's default values will suffice.

	// Set the gravity. Try changing this to see the effect on the objects
	gravity.Set(0.0f, -10.0f, 0.0f);

	// Ready to go, create the simulator object
	gSim = neSimulator::CreateSimulator(sizeInfo, NULL, &gravity);

	// Now we need to add the other elements to the physics engine, the
	// spheres, and the terrain.

	// Set the starting height for the first object and the additional
	// height increment for the subsequent objects
	fYPos = 4;
	fYPosAdd = SPHEREDIAM+0.2f;


	// Set the parameters for the spheres
	fMass = 100.0f;

	// Create some spheres
	for (i=0; i<SPHERECOUNT; i++)
	{
		gSpheres[i] = gSim->CreateRigidBody();
		geom = gSpheres[i]->AddGeometry();
		// Tell the geometry that we want a sphere and set its size
		geom->SetSphereDiameter(SPHEREDIAM);
		// Update the bounding information
		gSpheres[i]->UpdateBoundingInfo();
		// Set the inertia tensor -- again we use the sphere diameter here
		gSpheres[i]->SetInertiaTensor(neSphereInertiaTensor(SPHEREDIAM, fMass));
		gSpheres[i]->SetMass(fMass);
		// Slightly randomise the position to make it more interesting
		pos.Set((float)(rand()%25)-12, fYPos, (float)(rand()%25)-12);
		gSpheres[i]->SetPos(pos);

		// Set the index of the triangle into the UserData of the rigid body.
		// We can retrieve this in the Terrain callback in order to find
		// out which rigid body we're testing for.
		gSpheres[i]->SetUserData(i);


		// Increase the height for the next object
		fYPos += fYPosAdd;
	}

	// Build the terrain triangles for the floor.
	// Note that we don't call SetTerrainMesh() this time...
	BuildTerrainTriangles();
	
	//...but instead we call TerrainQueryCallback
	// to set our TerrainTriangleQueryCallback
	gSim->SetTerrainTriangleQueryCallback(TerrainQueryCallback);

	// All done
	return true;
}









void KillPhysics(void)
{
	if (gSim)
	{
		// Destroy the simulator.
		// Note that this will release all related resources that we've allocated.
		neSimulator::DestroySimulator(gSim);
		gSim = NULL;
	}
}



void BuildTerrainTriangles(void)
{
	int iTriIndex;
	float fTerrainX, fTerrainZ;
	int x,z;

	// First set the terrain node heights
	SetTerrainHeights();

	// Set the number of vertices into the triMesh
	gTriMesh.vertexCount = TERRAIN_VERTEXCOUNT;

	// Set the vertex values
	fTerrainZ = -TERRAINSIZE/2;
	for (z=0; z<TERRAINZ; z++)
	{
		fTerrainX = -TERRAINSIZE/2;
		for (x=0; x<TERRAINX; x++)
		{
			triVertices[x + z*TERRAINX].Set(fTerrainX, gfTerrainHeights[x][z], fTerrainZ);
			fTerrainX += (TERRAINSIZE / (TERRAINX-1));
		}
		fTerrainZ += (TERRAINSIZE / (TERRAINZ-1));
	}
	// Set the vertices into the triMesh
	gTriMesh.vertices = triVertices;


	// Set the number of triangles into the triMesh
	gTriMesh.triangleCount = TERRAIN_TRIANGLECOUNT;

	// Set the triangle values
	iTriIndex = 0;
	for (x=0; x<TERRAINX-1; x++)
	{
		for (z=0; z<TERRAINZ-1; z++)
		{
			// Make the triangle for the first half of the quad
			triData[iTriIndex].indices[0] = x     + z    *TERRAINX;
			triData[iTriIndex].indices[1] = x     + (z+1)*TERRAINX;
			triData[iTriIndex].indices[2] = (x+1) + z    *TERRAINX;
			triData[iTriIndex].materialID = 1;
			triData[iTriIndex].flag = neTriangle::NE_TRI_TRIANGLE;
			// Move to the next triangle in the array
			iTriIndex += 1;
			// Make the triangle for the second half of the quad
			triData[iTriIndex].indices[0] = x     + (z+1)*TERRAINX;
			triData[iTriIndex].indices[1] = (x+1) + (z+1)*TERRAINX;
			triData[iTriIndex].indices[2] = (x+1) + z    *TERRAINX;
			triData[iTriIndex].materialID = 1;
			triData[iTriIndex].flag = neTriangle::NE_TRI_TRIANGLE;
			// Move to the next triangle in the array
			iTriIndex += 1;
		}
	}
	// Set the triangles into the triMesh
	gTriMesh.triangles = triData;

	// That's the Tokamak bit all done, but in order to draw the terrain we need
	// to set up DirectX's geometry too. In order to keep the Tokamak and DirectX
	// as separate as possible I've implemented these in two functions. To avoid
	// doing all the calculation again, we'll read all the data from the triMesh
	// object. Call the other function now to update the DX geometry.
	BuildTerrainDX(&gTriMesh);
}




void BuildTerrainDX(neTriangleMesh *triMesh)
{
	short *pIndex = NULL;
	strVertex *pVertex = NULL;
	int iColour;
	HRESULT hr;
	int i, j;

	// First we'll lock the vertex buffer so that we can set its content
	hr=vbFloor->Lock(0, // Offset, we want to start at the beginning
					 0, // SizeToLock, 0 means lock the whole thing
					 (unsigned char **)&pVertex, // If successful, this will point to the data in the VB
					 0);  // Flags (none)
	if(FAILED(hr))
	{
		OutputDebugString("Error Locking vertex buffer (BuildTerrainDX)");
		return;
	}

	// To avoid redoing all the calculations already performed in BuildTerrainTriangles,
	// we'll just read all the values back from the supplied triMesh object.
	for (i=0; i<triMesh->vertexCount; i++)
	{
		// Set the position for this vertex
		pVertex->SetPoint(triMesh->vertices[i].v[0],
							triMesh->vertices[i].v[1],
							triMesh->vertices[i].v[2]);
		// Set the normal (for simplicity we'll set them all to face upwards)
		pVertex->SetNormal(0, 1, 0);

		// Set the colour. We'll use a fixed value for red and blue, and vary
		// the green component depending on the height of the vertex
		iColour = RGB(0x80, (int)(0x80 + (triMesh->vertices[i].v[1] * 20)) & 0xff, 0x80);
		pVertex->dwColour = iColour;

		// Move to the next vertex
		pVertex += 1;
	}

	// We're finished so unlock the vertex buffer
	vbFloor->Unlock();

	
	// Now we'll lock the index buffer so that we can set its content
	hr=ibFloor->Lock(0, // Offset, start at the beginning
					 0, // SizeToLock, 0 means lock the whole thing
					 (unsigned char **)&pIndex, // If successful, this will point to the data in the index
					 0);  // Flags (none)
	if(FAILED(hr))
	{
		OutputDebugString("Error Locking index buffer (BuildTerrainDX)");
		return;
	}
	
	// As before, we'll copy all the triangle data from the triMesh so we don't have to recalculate
	for (i=0; i<triMesh->triangleCount; i++)
	{
		// Copy the index values for each triangle
		for (j=0; j<3; j++)
		{
			pIndex[j] = triMesh->triangles[i].indices[j];
		}
		// Move the pIndex pointer to the next triangle
		pIndex += 3;
	}

	// We're finished so unlock the index buffer
	ibFloor->Unlock();

}



// Initialise the gfTerrainHeights array with our terrain node heights
void SetTerrainHeights(void)
{
	int x,z;

	// Loop for each terrain node
	for (x=0; x<TERRAINX; x++)
	{
		for (z=0; z<TERRAINZ; z++)
		{
			// Set the height for this node
			// We'll create a sin-based basin...
			gfTerrainHeights[x][z] = -( sinf((float)x/4) + sinf((float)z/4)) * 4 + 2;
			// ..and then add a slight random "ripple" to it to make it more interesting
			gfTerrainHeights[x][z] += (float)(rand()%20)/20;
		}
	}
}






void TerrainQueryCallback(const neV3 & minBound, const neV3 & maxBound,		// bounds of object to test
                                    s32         **candidateTriangles,		// indices of triangles to be tested
                                    neTriangle  **triangles,				// all possible test triangles
                                    neV3        **vertices,					// triangle vertices
                                    // 
                                    s32         *candidateCount,			// number of candidate triangles
                                    s32         *triangleCount,				// number of possible test triangles
                                    // 
                                    neRigidBody *rb)						// body to test
{

	static s32 sCandidateTriangles[TERRAIN_TRIANGLECOUNT];	// The indices of the triangles to be checked
	int iCandidateIndex = 0;
	int i, j;
	neV3 minTri, maxTri;
	float fThisX, fThisY, fThisZ;

	// Loop through each triangle in our terrain
	for (i=0; i<gTriMesh.triangleCount; i++)
	{
		// Set the min and max X and Z values to be the first point in the triangle
		fThisX = gTriMesh.vertices[gTriMesh.triangles[i].indices[0]].n.X;
		fThisY = gTriMesh.vertices[gTriMesh.triangles[i].indices[0]].n.Y;
		fThisZ = gTriMesh.vertices[gTriMesh.triangles[i].indices[0]].n.Z;
		minTri.Set(fThisX, fThisY, fThisZ);
		maxTri.Set(fThisX, fThisY, fThisZ);

		// Loop for the other two points in the triangle in order to find the
		// minimum and maximum coordinate values across all three vertices.
		for (j=1; j<3; j++)
		{
			fThisX = gTriMesh.vertices[gTriMesh.triangles[i].indices[j]].n.X;
			fThisY = gTriMesh.vertices[gTriMesh.triangles[i].indices[j]].n.Y;
			fThisZ = gTriMesh.vertices[gTriMesh.triangles[i].indices[j]].n.Z;
			if (fThisX < minTri.n.X) minTri.n.X = fThisX;
			if (fThisY < minTri.n.Y) minTri.n.Y = fThisY;
			if (fThisZ < minTri.n.Z) minTri.n.Z = fThisZ;
			if (fThisX > maxTri.n.X) maxTri.n.X = fThisX;
			if (fThisY > maxTri.n.Y) maxTri.n.Y = fThisY;
			if (fThisZ > maxTri.n.Z) maxTri.n.Z = fThisZ;
		}

		// Now we know the bounds of the triangle, do the bounds of the
		// rigid body fall within them?
		if (minBound.n.X <= maxTri.n.X && maxBound.n.X >= minTri.n.X && 
			minBound.n.Y <= maxTri.n.Y && maxBound.n.Y >= minTri.n.Y && 
			minBound.n.Z <= maxTri.n.Z && maxBound.n.Z >= minTri.n.Z)
		{
			// Yes, the rigid body is vertically in line with this triangle.

			// Add this triangle to the array of candidate triangles.
			sCandidateTriangles[iCandidateIndex] = i;
			iCandidateIndex += 1;

			// Use the GetUserData method to retrieve the sphere index
			// we previously placed into the rigid body. We'll use this
			// as part of a binary shift so that each rigid body sets
			// a corresponding bit in the giTriangleActive array.
			giTriangleActive[i] |= 1 << rb->GetUserData();
		}

	}

	// Now we've built the array of candidate triangles,
	// we return values back to Tokamak as follows:
	*candidateTriangles = sCandidateTriangles;	// The array of candidate triangles
	*triangles = gTriMesh.triangles;	// The array of all possible triangles
	*vertices = gTriMesh.vertices;	// The array of vertices
	*candidateCount = iCandidateIndex;	// The number of candidate triangles we're returning
	*triangleCount = gTriMesh.triangleCount;	// The number of all possible triangles we're returning

	// All done!
}









// Initialise a timer for use in our application.
// Wherever possible use a Performance timer as this
// gives the best accuracy.
// If this isn't available, fall back to the multimedia timer.
// In each case, note the number of "ticks" per second so that
// we can divide the elapsed time by this value to determine
// how much real time has actually passed.
// For more information on using these timers, see the articles at
// www.mvps.org/directx
bool InitTimer(void)
{
	INT64 TimerFrequency;

	// Can we use the high performance timer?
	if (QueryPerformanceFrequency( (LARGE_INTEGER*)&TimerFrequency))
	{
		// High performance timer available, let's get ready to use it
		gfTimeScale = 1.0f / TimerFrequency;
		QueryPerformanceCounter((LARGE_INTEGER *) &gCurrentTime);
		gbUseHFTimer = true;
		return true;
	}
	else
	{
		// No high precision timer to be seen, let's use the Windows multimedia timer instead
		gfTimeScale = 0.001f;
		gCurrentTime = timeGetTime();
		gbUseHFTimer = false;
		return true;
	}

	// No timer available at all (should never happen)
	return false;
}


// Use the timer we previously initialised to determine
// how much time (in seconds, or a fraction thereof) has
// passed since this function was last called.
float GetElapsedTime()
{
	INT64 newTime;
	float fElapsed;

	if (gbUseHFTimer) 
	{
		QueryPerformanceCounter((LARGE_INTEGER *) &newTime);
	}
	else 
	{
		newTime=timeGetTime();
	}

	// Scale accordingly
	fElapsed = (float)((newTime - gCurrentTime) * gfTimeScale);

	// Save the new time value for the next time we're called
	gCurrentTime = newTime;

	return fElapsed;
}




// Windows message processor
LRESULT CALLBACK WindowProc(HWND hWnd, UINT Msg, WPARAM WParam, LPARAM LParam)
{
	HDC hDc;
	PAINTSTRUCT ps;
   
	switch(Msg)
	{
		case WM_KEYDOWN:  // A key has been pressed, end the app
			gbFinished = true;
			return 0;

		case WM_PAINT:    //Repaint our window
			hDc=BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			return 0;

		case WM_CLOSE:    //User hit the Close Window button, end the app
			gbFinished = true;
			return 0;

		case WM_DESTROY:  //This window is being destroyed, tell Windows we're quitting
			PostQuitMessage(0);
			return 0;
	}

	return DefWindowProc(hWnd, Msg, WParam, LParam);
}



// If the DirectX device is lost for some reason (e.g.,
// the user presses Alt-Tab in a full-screen app),
// we'll use this function to determine whether the
// device is ready to resume its previous display
// processing. If so we'll reinitialise everything
// and return true, otherwise we'll return false.
bool ValidateDevice(void)
{

	HRESULT hr;

	hr = gD3DDevice->TestCooperativeLevel();
	if (FAILED(hr))
	{
		// If the device is lost, return False
		if (hr == D3DERR_DEVICELOST) return false;

		// If the device is waiting to be reset,
		// reset it, reinitialise the scene and return
		// true.
		if (hr == D3DERR_DEVICENOTRESET)
		{
			gD3DDevice->Reset(&gD3Dpp);

			InitScene();

			return true;
		}
	}

	// The device is still working normally, nothing to do.
	return true;
}






void BuildSphere(IDirect3DVertexBuffer8 **pNewBuff, float fDiameter)
{
	HRESULT hr;
	strVertex *strVertexData;

	int iVertexCount;
	float fX1, fY1, fX2, fY2, fX3, fY3, fX4, fY4;	// The vertex positions around each quad we calculate
	float fAngle, fAngleAdd;	// The angle around the X axis of the sphere
	float fY, fYNext;
	float fSineAngle, fSineAdd;	// The angle around the Y axis of the sphere
	int i, j;
	int v;

	float fNormalCalcPoints[3][3];
	float fNormal[3];

	float fRadius;
	
	// Calculate the number of vertices required to create out sphere
	iVertexCount = (SPHERE_HSTRIPS+1) * SPHERE_VSLICES * 6;

	// Calculate the radius of the sphere
	fRadius = fDiameter / 2;

	// Create an empty vertex buffer
	BuildVertexBuffer(pNewBuff, NULL, iVertexCount);

	// Lock the vertex buffer ready to receive our data
	hr=(*pNewBuff)->Lock(0, 0, (unsigned char**)&strVertexData, 0);
	if(FAILED(hr))
	{
		OutputDebugString("Error Locking vertex buffer (BuildSphere)");
		exit(5);
	}

	// Calculate the angle to add each time as we divide the cylined into the appropriate number of strips
	fAngleAdd = 360.0f / (float)SPHERE_VSLICES;
	fSineAngle = 0;
	fSineAdd = 180.0f / (SPHERE_HSTRIPS-1);

	// Loop around our sphere
	for (i=0; i<SPHERE_HSTRIPS; i++)
	{

		// Reset the angle for this slice
		fAngle = 0;
		fY = cosf(fSineAngle * DEG2RAD) * fRadius;
		fYNext = cosf((fSineAngle+fSineAdd) * DEG2RAD) * fRadius;

		for (j=0; j<SPHERE_VSLICES; j++)
		{
			// Calculate the X and Y position for the sphere (as if it were a circle viewed from above)
			fX1 = sinf(fAngle * DEG2RAD) * fRadius * sinf(fSineAngle * DEG2RAD);
			fY1 = cosf(fAngle * DEG2RAD) * fRadius * sinf(fSineAngle * DEG2RAD);
			fX2 = sinf((fAngle+fAngleAdd) * DEG2RAD) * fRadius * sinf(fSineAngle * DEG2RAD);
			fY2 = cosf((fAngle+fAngleAdd) * DEG2RAD) * fRadius * sinf(fSineAngle * DEG2RAD);
			fX3 = sinf(fAngle * DEG2RAD) * fRadius * sinf((fSineAngle + fSineAdd) * DEG2RAD);
			fY3 = cosf(fAngle * DEG2RAD) * fRadius * sinf((fSineAngle + fSineAdd) * DEG2RAD);
			fX4 = sinf((fAngle+fAngleAdd) * DEG2RAD) * fRadius * sinf((fSineAngle + fSineAdd) * DEG2RAD);
			fY4 = cosf((fAngle+fAngleAdd) * DEG2RAD) * fRadius * sinf((fSineAngle + fSineAdd) * DEG2RAD);

			// Set the first triangle
			strVertexData[0].SetPoint(fX1, fY    , fY1);
			strVertexData[1].SetPoint(fX4, fYNext, fY4);
			strVertexData[2].SetPoint(fX2, fY    , fY2);
			// Set the second triangle
			strVertexData[3].SetPoint(fX1, fY    , fY1);
			strVertexData[4].SetPoint(fX3, fYNext, fY3);
			strVertexData[5].SetPoint(fX4, fYNext, fY4);

			// Now we need to calculate the normal for this surface for lighting
			// Read out the position for the first point in our triangle
			fNormalCalcPoints[0][0] = strVertexData[4].fX;
			fNormalCalcPoints[0][1] = strVertexData[4].fY;
			fNormalCalcPoints[0][2] = strVertexData[4].fZ;
			// Read out the position for the second point in our triangle
			fNormalCalcPoints[1][0] = strVertexData[1].fX;
			fNormalCalcPoints[1][1] = strVertexData[1].fY;
			fNormalCalcPoints[1][2] = strVertexData[1].fZ;
			// Read out the position for the third point in our triangle
			fNormalCalcPoints[2][0] = strVertexData[0].fX;
			fNormalCalcPoints[2][1] = strVertexData[0].fY;
			fNormalCalcPoints[2][2] = strVertexData[0].fZ;

			// Calculate the normal
			CalcNormal(fNormalCalcPoints, fNormal);

			// Colour the sphere and set its face normals
			for (v=0; v<6; v++)
			{
				strVertexData[v].SetNormal(fNormal[0], fNormal[1], fNormal[2]);

				if ((i+j)%2 == 0)
					strVertexData[v].dwColour = 0xffff6666;
				else
					strVertexData[v].dwColour = 0xffffffff;
			}

			// Move to the next pair of triangles
			strVertexData += 6;
		
			// Move to the next angle
			fAngle += fAngleAdd;
		}

		fSineAngle += fSineAdd;
	}

	//Unlock so Direct3D knows we're done and can do any behind-the-scenes magic required
	(*pNewBuff)->Unlock();

}







// Here comes the graphics bit.
void Render(void)
{
	float fElapsed;
	static float fLastElapsed;
	int iFloorColour;
	neT3 t;
	D3DXMATRIX dxTrans;
	int i;
	// We'll use the following three objects to reset spheres when the "fall off" the floor
	neV3 pos, vel, spin;


	// Ensure the DirectX device is still ok. If not, sleep for a couple
	// of seconds (to avoid using 100% CPU when we've not got the focus)
	// and then stop until the next time we're called.
	if (!ValidateDevice())
	{
		Sleep(100);
		GetElapsedTime();
		return;
	}

	// Find out how much time has elapsed since we were last called
	fElapsed = GetElapsedTime();

	// In order to keep passing a reasonably stable value into the simulator's Advance
	// method, we'll try to stop occasional variations in the elapsed time from occurring.
	// We'll do this by preventing the elapsed time from being more than 20% greater or
	// less than the previous elapsed time.
	if (fLastElapsed != 0)
	{
		if (fElapsed > fLastElapsed * 1.2f) fElapsed = fLastElapsed * 1.2f;
		if (fElapsed < fLastElapsed * 0.8f) fElapsed = fLastElapsed * 0.8f;
	}
	// To avoid particularly large spikes, we'll stop the elapsed time from exceeding
	// 1/45th of a second. Values larger than this can lead to an unstable simulation.
	if (fElapsed > 1.0f / 45.0f) fElapsed = 1.0f / 45.0f;
	// Store the elapsed time so that we can use it the next time around
	fLastElapsed = fElapsed;



	// Clear out the TriangleActive array so that the Terrain Callback can update it.
	for (i=0; i<TERRAIN_TRIANGLECOUNT; i++)
	{
		giTriangleActive[i] = 0;
	}

	// Tell the Tokamak simulator to advance all its calculations by one frame.
	// Note that we pass the elapsed time here so that the simulation speed
	// is constant regardless of the refresh frequency, frame rate, etc.
	gSim->Advance(fElapsed);


	// To make things look more interesting, we'll rotate the camera
	// around the simulation area. Remove these statements if you
	// prefer a fixed camera position
	D3DXMATRIX view_matrix;
	D3DXMatrixLookAtLH(&view_matrix,&D3DXVECTOR3( sinf(gfCameraAngle*0.2f)*15, 25.0f, cosf((gfCameraAngle+0.2f)*0.25f)*15 ),
							   &D3DXVECTOR3( 0.0f, 0.0f, 0.0f ),
							   &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));
	gD3DDevice->SetTransform(D3DTS_VIEW,&view_matrix);




	D3DMATERIAL8 myMat;
	ZeroMemory( &myMat, sizeof(D3DMATERIAL8) );
	myMat.Diffuse.r = myMat.Ambient.r = 1.0f;
	myMat.Diffuse.g = myMat.Ambient.g = 1.0f;
	myMat.Diffuse.b = myMat.Ambient.b = 1.0f;
	myMat.Diffuse.a = myMat.Ambient.a = 1.0f;
	gD3DDevice->SetMaterial( &myMat );



	//Clear the screen and Z-buffer (make the screen a nice blue colour)
	gD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER , D3DCOLOR_XRGB(64,64,128), 1.0f, 0 );

	// Notify DX that we're ready to render
	if(SUCCEEDED(gD3DDevice->BeginScene()))
	{


		// Set the vertex stream for the spheres
		gD3DDevice->SetStreamSource(0,vbSphere,sizeof(strVertex));
		// Draw the spheres
		for (i=0; i<SPHERECOUNT; i++)
		{
			// Get the transformation matrix for this cube
			t = gSpheres[i]->GetTransform(); 
			// Transfer the values to a D3DMATRIX that we can pass to DirectX
			dxTrans = D3DXMATRIX(	t.rot[0][0], t.rot[0][1], t.rot[0][2], 0.0f, 
									t.rot[1][0], t.rot[1][1], t.rot[1][2], 0.0f, 
									t.rot[2][0], t.rot[2][1], t.rot[2][2], 0.0f, 
									t.pos[0],t.pos[1], t.pos[2], 1.0f );
			// Set the world transformation so that we can draw the cube at the correct position
			gD3DDevice->SetTransform(D3DTS_WORLD, &dxTrans );
			// Render the cube
			gD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST,0,SPHERE_TRIANGLECOUNT);

			// Check the Y-position of the sphere.
			// If it falls too low then the sphere has fallen off of the floor;
			// we'll re-drop it if this occurs to keep things moving.
			if(t.pos[1] < -25)
			{
				// Reset the position of the sphere
				pos.Set((float)(rand()%10)-4, 20, (float)(rand()%10)-4);
				gSpheres[i]->SetPos(pos);
				// Reset the velocity (so it doesn't already have a downward momentum from before)
				vel.Set(0,0,0);
				gSpheres[i]->SetVelocity(vel);
				// Reset the angular momentum (so it isn't spinning from before);
				spin.Set(0,0,0);
				gSpheres[i]->SetAngularMomentum(spin);
			}

		}


		// Set the vertex and index stream for the terrain
		gD3DDevice->SetStreamSource(0,vbFloor,sizeof(strVertex));
		gD3DDevice->SetIndices(ibFloor, 0);
		// Reset the view matrix to the identity matrix
		dxLoadIdentity();
		dxApplyMatrix(gD3DDevice);
		// Draw the terrain
		for (i=0; i<TERRAIN_TRIANGLECOUNT; i++)
		{
			// Assume the ambient light will be the "normal" colour
			iFloorColour = 0x00202020;
			// Set the red, green or blue components of the ambient
			// light to full strength if one of our spheres is over
			// this triangle so that we can highlight it.
			if (giTriangleActive[i] & 1) iFloorColour |= 0x00ff0000;
			if (giTriangleActive[i] & 2) iFloorColour |= 0x0000ff00;
			if (giTriangleActive[i] & 4) iFloorColour |= 0x000000ff;

			// Set the ambient light so that this floor triangle is rendered
			// in the colour we've defined
			gD3DDevice->SetRenderState(D3DRS_AMBIENT,iFloorColour);

			// Set the vertex and index stream for the terrain
			gD3DDevice->SetStreamSource(0,vbFloor,sizeof(strVertex));
			gD3DDevice->SetIndices(ibFloor, 0);
			// Draw this triangle
			gD3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
											0, TERRAIN_VERTEXCOUNT,
											i*3, 1);
		}

		// Reset ambient lighting for subsequent rendering
		gD3DDevice->SetRenderState(D3DRS_AMBIENT,0x00202020);


		//Notify the device that we're finished rendering for this frame
		gD3DDevice->EndScene();
	}

	//Show the results
	gD3DDevice->Present( NULL, NULL, NULL, NULL );


	// Update the camera angle
	gfCameraAngle += fElapsed;

	gfSceneTime += fElapsed;

	// Periodically reset the object positions
	if (gfSceneTime > 20)
	{
		KillPhysics();
		InitPhysics();
		gfSceneTime = 0;
	}

	// All done!
}
